/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2013 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Philippe Merle
 */

package org.ow2.frascati.tinfi.opt.oo;

import java.lang.reflect.Modifier;

import org.oasisopen.sca.ServiceReference;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.Utils;
import org.objectweb.fractal.juliac.api.ClassGeneratorItf;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.BlockSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.ow2.frascati.tinfi.TinfiComponentOutInterface;

/**
 * This class generates the source code of component client (output) interface
 * implementations.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Philippe Merle <Philippe.Merle@inria.fr>
 */
public class ClientInterfaceClassGenerator
extends ServerInterfaceClassGenerator {
    
    public ClientInterfaceClassGenerator(
        Juliac jc,
		InterfaceType it, MembraneDesc<?> membraneDesc, boolean mergeable ) {
        
        super(jc,it,membraneDesc,mergeable);
    }
    
    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    @Override
    public String getClassNameSuffix() {
        return "FcOutItf";
    }

    @Override
    public String getSuperClassName() {
        
    	/*
         * The generated client interface extends the class implementing the
         * server interface of the same type.
         */
        ClassGeneratorItf cg =
            new ServerInterfaceClassGenerator(jc,it,membraneDesc,mergeable);
        String name = cg.getTargetClassName();

        /*
         * Add type parameters if any.
         */
        UnifiedClass proxycl = jc.create(it.getFcItfSignature());
        String[] tpnames = proxycl.getTypeParameterNames();
        if( tpnames.length != 0 ) {
            String s = Utils.getTypeParameterNamesSignature(tpnames);
            name += s;
        }
        
        return name;
    }

    @Override
    public String[] getImplementedInterfaceNames() {
        
        String[] old = super.getImplementedInterfaceNames();
        String[] result = new String[old.length+1];
        System.arraycopy(old,0,result,0,old.length);
        
        StringBuffer sb = new StringBuffer();
        sb.append(TinfiComponentOutInterface.class.getName());
        sb.append('<');
        sb.append(it.getFcItfSignature());
        sb.append('>');
        result[old.length] = sb.toString();
        
        return result;
    }
    
    @Override
    public void generateMethods( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
        
        super.generateMethods(cv, proxycl);
        
        BlockSourceCodeVisitor mv =
            cv.visitMethod(
                Modifier.PUBLIC, null,
                ServiceReference.class.getName()+'<'+it.getFcItfSignature()+'>',
                "getServiceReference", null, null );
        ClassGeneratorItf cg =
            new ServiceReferenceClassGenerator(jc,it,membraneDesc,mergeable);
        String srclname = cg.getTargetClassName();
        mv.visit  ("    return new ");
        mv.visit  (srclname);
        mv.visit  ("(");
        mv.visit  (it.getFcItfSignature());
        mv.visitln(".class,this);");
        mv.visitEnd();
    }    

    @Override
    public void generateProxyMethod(
        ClassSourceCodeVisitor cv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        /*
         * Indeed nothing.
         * Proxy methods are inherited from the component server interface
         * class.
         */
    }
}
